---
author: François Pelletier
title: Notes de cours - examen final
fig_caption: yes
highlight: pygments
number_sections: yes
toc: yes
toc_depth: 3
geometry: "left=2cm,right=2cm,top=2cm,bottom=2cm"
fontsize: 12pt
---

# Extraction d'information

Objectif: obtenir de l'information utile et la structurer.

Usages:

- Compréhension de textes
- Systèmes questions-réponses
- Traductions automatiques

Approches:

- À base de règles
- Par apprentissage

## Types d'information:

- Quasi structurée: texte préformaté, par exemple un bulletin météo
- Semi structurée: texte en points de forme, par exemple une annonce classée
- Non structurée: texte libre, par exemple un article de nouvelles

## Types d'extraction

- Reconnaissance d'entitées nommées
- Détection et classification de relations
- Détection et classification d'évènements

## Reconnaissance d'entitées nommées

**Entité nommée**: Se désigne par un nom propre, une quantité, une date, une marque temportelle ou un montant d'argent. Peuvent être spécifiques au domaine d'application

Objectif: Trouver les entitées nommées et les classer selon leur type. 

Usages:

- Indexation
- Identification de relations
- Résolution de requêtes
- Lien avec une base de connaissances

Exemples de types d'entités nommées:

- Personne
- Marques de commerce
- Lieu
- Organisation
- Géopolitique
- Argent
- Temps

Enjeux: ambiguité du type des entités. Peuvent appartenir à plusieurs catégories

### Approches de NER

#### Apprentissage automatique:

Encodage IOB mot par mot:

- **B**eginning (premier jeton de l'entité)
- **I**nside (autres jetons de l'entité)
- **O**utside (ne fait pas partie d'une entité nommée)

Chaque étiquette a une position et un type, pour un total de $2n+1$ étiquettes possibles.

Étiquetage séquentiel:

- MEMM/CRF (généralisation de Viterbi)
- bi-LSTM (réseau de neurones récurrent et bidirectionnel)
- HMM (Modèle de Markov caché)
- SVM (Modèle discriminant)

Extraction des attributs des mots:

- identité, plongement, POS du mot et des voisins
- Étiquette syntaxique du mot, du groupe de mot et des mots voisins
- Mot présent dans un index géographique
- Contient un préfixe ou un suffixe
- Tout en majuscules
- Forme du mot et des voisins
- Forme du mot court et des voisins
- Présente de trait ou de tiret

Portée d'un classificateur MEMM:

![Diagramme MEMM pour NER](memm-ner.png)

#### Base de règles

Caractéristiques:

- Largement utilisé en pratique
- Fort niveau de précision

Composantes:

- Listes d'entités
- Patrons ou expressions régulière pour reconnaitre des entités potentielles
- Patrons pour reconnaitre les environnements où des entités peuvent se trouver

Approche hybride:

1. Identifier les attributs:
    - Règles pour repérer les entités non-ambigues
    - Comparer des sous-chaines avec des entités connues
    - Listes de noms d'entités spécifiques
2. Classificateur de séquence

## Détection et classification de relations

Objectif: capturer les relations sémantiques du domaine d'application, populer une base de données

Formes des relations:

ENTITÉ - RELATION - ENTITÉ

Types de relations:

- PERSON-SOCIAL
    - Family
    - Business
    - Lasting personal
- PHYSICAL
    - Near
    - Located
- GENERAL AFFILIATION
    - Citizen
    - Resident
    - Ethnicity
    - Religion
    - Org-Location-Origin
- PART-WHOLE
    - Subsidiary
    - Geographical
- ORG AFFILIATION
    - Founder
    - Ownership
    - Membership
    - Sports affiliation
    - Employment
    - Student-Alumni
    - Investor
- ARTIFACT
    - User
    - Owner
    - Inventor
    - Manufacturer

### Exemple

| Domain | D = {a, b, c, d, e, f , g, h, i} |
|----------------------------------------------------------|--------------------------------------------|
| United, UAL, American Airlines, AMR | a, b, c, d |
| Tim Wagner | e |
| Chicago, Dallas, Denver, and San Francisco | f , g, h, i |


| Classes |  |
|----------------------------------------------------------|--------------------------------------------|
| United, UAL, American, and AMR are organizations | Org = {a, b, c, d} |
| Tim Wagner is a person | Pers = {e} |
| Chicago, Dallas, Denver, and San Francisco are places  | Loc = { f , g, h, i} |


| Relations |  |
|----------------------------------------------------------|--------------------------------------------|
| United is a unit of UAL | PartOf = {ha, bi, hc, di} |
| American is a unit of AMR |  |
| Tim Wagner works for American Airlines | OrgAff = {hc, ei} |
| United serves Chicago, Dallas, Denver, and San Francisco | Serves = {ha, f i, ha, gi, ha, hi, ha, ii} |

### Ressources

- Wikipedia
    - Infobox
- DBPedia
    - Représentation structurée de données provenant de Wikipedia
- Freebase
    - Personnes, lieux et objets
- Wordnet
    - Différentes relations sémantiques

### Approches possibles

- Règles écrites à la main
    - Forte précision
    - Faible rappel
    - Utiles pour un domaine spécifique, demande du temps pour la création des règles
- Apprentissage supervisé
    - Corpus d'entraînement étiqueté avec des entités et des relations
    - Classificateur pour chaque paire d'entités: 
        - Vérifier si reliées (modèle 1, exemples positifs et négatifs), 
        - Puis classifier la relation (modèle 2)
        - Régression logistique, Naïve Bayes, SVM
    - Attributs:
        - Prémisse (headword)
        - Mots précédant M1
        - Mots suivant M2
        - Mots entre M1 et M2
        - Type d'entité de M1
        - Type d'entité de M2
        - Concaténation des types
        - Analyse syntaxique des constituants
        - Analyse par morceaux (chunk) des constituants
        - Analyse des dépendances syntaxiques
- Apprentissage semi-supervisé
    - Classifier une relation sans corpus étiqueté
    - Bootstrapping (répéter plusieurs fois):
        - Quelques exemples
        - Extraire les paires d'entités de ces exemples
        - On les recherche dams un corpus ou sur le web
        - On apprend des patrons qui généralisent le contexte
        - On recherche ces patrons et on conserve ceux qui donnent plusieurs résultats
    - Semantic drift
        - Exemples erronés
        - Niveau de confiance:

        $$
        Conf(p) = \frac{hits_p}{finds_p} log(finds_p)
        $$

        - Niveau de confiance d'un exemple: 

        $$
        Conf(t) = 1-\prod_{p \in P}(1-Conf(p))
        $$

- Supervision distante
    - Utiliser une ressource avec un grand nombre d'exemples de qualité (DBPedia, Freebase, WordNet)
    - Trouver des phrases avec les entités de cette ressource
    - Extraire des exemples d'entraînement
    - Classificateur supervisé (avec les mêmes types d'attributs que l'approche supervisée)
    - Exemples négatifs: paires d'entités tirés au hasard qui n'apparaissent pas dans les relations
    - Avantages: mêmes que ceux des approches précédentes
    - Désavantage: Peu de relations pour de nouveaux domaines
- Apprentissage non supervisé
    - Patrons syntaxiques pour les relations:
    ```
    V | VP | VW*P
    V = verb particle? adv?
    W = (noun|adj|adv|pron|det)
    P = (prep|particle|inf. marker)
    ```
    - Segmenter les textes en phrases
    - Pour chaque phrase:
        - Faire un POS tagging et une analyse syntaxique
        - Trouver la plus longue séquence qui satisfait un patron à partir d'un verbe
        - Trouver les groupes nominaux à gauche et à droite
        - Déterminer un niveau de confiance en fonction de la fréquence des relations
    - Avantage: pas besoin de connaître les relations
    - Désavantage: On doit les convertir en relations sémantiques pour les stocker. Les relations sont basées sur des verbes, on va manquer les autres types de relations.

## Autres extractions

### Extraction d'expressions temporelles

- Similaire à l'extraction d'entitées nommées
- Doivent être normalisées. Peuvent être absolues, relatives ou sous la forme de durées.

### Détection et classification d'évènements

- Repérage des verbes d'actions, d'état ou de communication
- Classificateurs de séquences
- Tri chronologique des évènements, en utilisant aussi l'extraction d'expressions temporelles

# Systèmes questions-réponses

Types de questions:

- Factuelles (Qui, où) ?
- Complexes (Que pense, Comment) ?

![Système QA factuel](systeme-qa-factuel.png)

## Approche avec un corpus de texte

- Réponses dans des documents
- Recherche d'information (moteur de recherche et indexation)

### Traitement des questions

- Détection du type de réponse
- Formulation de requête pour le moteur de recherche d'information
- Information à extraire de la question:
    - Type de réponse
    - Focus
    - Type de question
    - Mots-clés pour la requête

### Extraction des termes de requête

- Termes de requête: pour repérer des passages. 
Exclure:
    - le mot-question, 
    - les mots-outils et 
    - les verbes fréquents
- Taille du corpus: une seule réponse, formulation différente
- Expansion de requête: 
    - variantes morphologiques (mots de la même famille)
    - synonymes (provenant d'un thesaurus)
- Reformulation:
    - Forme affirmative, avec règles de réécriture (phrase ou le mot manquant est la réponse)
    - Ex: 
    ```
    wh-word did A verb B -> A verb +ed B
    Where is A -> A is located in
    ```

### Détection du type d'entité de la réponse

- Permet de limiter les passages pertinents et se concentrer sur les bonnes phrases.
- **Limitations**: les types d'entités détectés par les logiciels NER sont limités
- Taxonomie de réponse à deux niveaux: Li&Roth
    - 6 grandes classes
    - 50 sous-classes

Deux approches:

- Règles manuelles (expressions régulières)
- Classification automatique:
    - Annotation des données d'entraînement
    - Classification avec ces attributs
        - Mots de la question
        - POS
        - Entités nommées
        - Mot principal (headword), du premier groupe nominal après le mot question

### Traitement de passages

- Résultat de la recherche: liste de documents priorisés selon leur similarité
- Problèmes:
    - Les documents ne sont pas les meilleures unités
    - Les documents en haut de la liste ne contiennent pas nécessairement la réponse
- Solution:
    - Découper les extraits des documents (segmantation)
        - Section, paragraphe, phrase
        - Fenêtre de mots
    - Filtrer les extraits qui ne semblent pas contenir de réponse
    - Réordonner ces extraits (avec des attributs)
        - \# d'entités nommées du bon type
        - \# de mots clés de la requête
        - Plus longue séquence de mots de la question
        - Rang du document d'où le passage provient
        - Proximité des mots-clés de la requête
        - Nombre de N-grammes en commun
    - Sur le web, utiliser les "snippets"

### Extraction de réponses

- Span labeling: identifier la portion du texte qui constitue la réponse
    - Approche: NER, retourner la chaîne du bon type
    - Si la réponse n'est pas une entité nommée, on utilise des règles. Ces règles peuvent être apprises (comme pour l'extraction de relations, par exemple)
- Si plusieurs réponses: on propose les différentes réponses ou on utilise un classificateur
    - Type de la réponse
    - Correspondance à un patron
    - Distance du mot-clé
    - Mot observé pour la première fois
    - Apposition (nombre de mots entre le mot-question et la réponse, avec un maximum de 3 mots et une virgule)
    - Position des ponctuations
    - Séquence de mots de la question
- Mean reciprocal rank

$$
MRR = \frac{1}{N}\sum_{i=1}^N \frac{1}{rank_i} 
$$

## Approche avec des données structurées

- Base de connaissances (entités et relations, dbPedia, Freebase, WordNet). Extraction d'information
- Ex: Siri, Wolfram Alpha

# Modèles statistiques de traduction automatique

Traduction statistique: difficile de trouver les ressources autrement

- Corpus bitextes (même textes dans deux langues)
- Trouver les mots ou les groupes de mots qui vont ensemble
- Parfois difficile d'avoir une bonne traduction, le sens des mots est différent entre les langues.
- **modèle IBM**: Compromis entre **fidélité** et **fluidité**

| Fidélité              | Fluidité                |
|-----------------------|-------------------------|
| sens de la traduction | lisibilité              |
| même information      | respect de la grammaire |
|                       | qualité du texte        |

Mathématiquement:

$$
\hat{T} = argmax_T fidelite(T,S) fluidite(T)
$$

En probabilités, on obtient:

$$
\hat{T} = argmax_T P(T|S) = argmax_T \frac{P(S|T)P(T)}{P(S)} = argmax_T P(S|T)P(T)
$$

où $P(S|T)$ est un modèle de traduction et $P(T)$ un modèle de langue.

## Canal bruité

![Traduction avec l'analogie du canal bruité](traduction-canal-bruite.png)

## Modèle de langue

- Modèles N-grammes (jusqu'à 5-grammes en anglais)
- Modèles syntaxiques (dépendance plus longues entre les mots)

## Modèle de traduction

Permet de quantifier les meilleures traductions pour un segment de phrase ou une phrase complète.

### Mots individuels

![Traduction de mots individuels](traduction-mots-invid.png)

- Les mots d'une phrase sont des traductions plausibles des mots de l'autre phrase

### Par séquence de mots (phrase-based)

![Traduction par séquence de mots](traduction-phrase-based.png)

- Inclus aussi des mots individuels
- Donne de meilleurs résultats que des mots individuels
- 3 étapes:
    - Regroupes les mots en groupes $E=\bar{e}_1, \bar{e}_2, \ldots$ (segmentation)
    - Traduire chaque groupe $\bar{e}$ en $f_i$:  $\phi(f_i|\bar{e}_i)$
    - Déplacer les groupes

$$
P(F|E) = \prod_{i=1}^I \phi(\bar{f}_i,\bar{e}_i)d(i)\\
$$

- **Distorsion**: Distance à laquelle le groupe est déplacé
- **Probabilité de distortion** $d(i)$ du ieme groupe. Distance entre $a_i$, la position de $f$ généré par $\bar{e}_i$ et $b_{i-1}$, la position de $f$ généré par le groupe précédent $\bar{e}_{i-1}$

$$
d(i)=\alpha^{|a_i-b_{i-1}-1|},\,0<\alpha<1
$$

- **Probabilités de traduction**: $\phi(f_i|\bar{e}_i)$
    - Corpus parallèle, avec correspondance entre les groupes de $E$ et $F$
    - Maximum de vraisemblance des probabilités
        - Compte des fréquences:

        $$
        \bar\phi(f_i|\bar{e}_i) = \frac{count(\bar{f}_i|\bar{e}_i)}{\sum_{\bar{f}}count(\bar{f}_i|\bar{e}_i)}
        $$

        - Corpus d'alignements de groupes de mots: difficile à construire.
        - On utilise les **alignements de mots**: Association des mots sources aux mots cibles dans des phrases parallèles.

        ![](traduction-alignement-mots.png)

        - Alignement un à plusieurs: Chaque mot cible a un seul mot source. Représentation sous forme de vecteur
        - Mots parasites: générés par un mot nul $e_0$
        - L'algorithme d'alignement donne les paramètres du modèle $P(F,E)$

## Modèle IBM 1

- Approche générative: $E \rightarrow F$
- Choix de la longueur $J$ de $F$: $F=f_1,f_2,\ldots,f_j$
- Alignement 1 à plusieurs
- Pour chaque position de Fm générer $f_j$ correspondant à $e_{aj}$

### Calcul de $P(F|E)$

- $(I+1)^J$ alignements possibles, incluant le *mot nul*
- Distribution de longueur $\epsilon=P(J|E)$
- Alignements équiprobables:

$$
P(A|E) = P(A|E,J)P(J|E) = \frac{\epsilon}{(I+1)^J}
$$

- Soit $t(f_x,e_y)$ la probabilité de traduire $e_y$ en $f_x$:

$$
P(F|E,A) = \prod_{j=1}^J t(f_i,e_{a_j})
$$

- $P(F|E)$ est la somme sur tous les alignements

$$
P(F|E) = \sum_A P(F|E,A)P(A|E) = \sum_{a \in A} \frac{\epsilon}{(I+1)^J} \prod_{j=1}^J t(f_i,e_{a_j})
$$

## Apprentissage des alignements

- Corpus parallèles (Hansard canadien, Hong Kong Hansards)
- Alignement des phrases
    - Longueur en mots ou en caractères
    - Programmation dynamique
    - Modèle simple de traduction
    - Retrait des phrases non alignées, fusion des phrases source des alignements qui ont la même phrase dans la destination (n à m, $n,m>0$)
- Alignement des mots dans les phrases
    - Bootstrapping
    - Algorithme Expectation-Maximization
        - E: probabilité de tous les alignements possibles (paramètres courants du modèle)
        - M: réestimer les valeurs de chaque paramètre
- Alignement des groupes de mots
    - Application de la symétrie à partir des alignements de mots, en faisant l'intersection des alignements pour $E \rightarrow F$ et $F \rightarrow E$. On ajoute des alignements en utilisant l'union (Algorithme de Och and Ney), avec des critères tels que:
        - case vide,
        - adjacence,
        - probabilité
        
    - Table de traduction: on conserve les groupes de mots pour le décodage

## Décodage

Obtenir la meilleure traduction E depuis la phrase F

$$
argmax_{E \in English} P(F|E)P(E)
$$

- On ne peut enumérer toutes les phrases possibles
- Algorithme d'exploration (stack decoding, A*)
    - $f(s) = g(s)+h(s)$: coût actuel (partiel) + coût estimé. Optimal si $h(s)$ ne surestime pas, donc est admissible

    ![Stack decoding - A*](stack-decoding.png)

- On considère seulement les groupes possibles avec la table de traduction
- Coût actuel $g(s)$: probabilité totale pour les groupes traduits

$$
cost(E,F) = \prod_{i \in S} \phi(\bar{f}_i,\bar{e}_i) d(a_i-b_{i-1}) P(E)
$$

- Coût futur
    - Le coût de traduire les mots restants dans la phrase source
    - Très coûteux à évaluer précisément
    - Approximation:
        - Pas de distortion 
        - Minimiser le produit du modèle de traduction et de langue
        - Séquence la plus probable par Viterbi
- Q devient rapidement trop grand
    - Recherche par faisceau: on conserve les $k$ solutions les plus prometteurs à chaque étape.
- Difficile de comparer le coût pour un nombre de mots différents
    - Solution possible: Une file par nombre de mots traduits
    - Multistack 
    
## Évaluation

- Évaluation humaine: la plus fiable
- Évaluation automatique: moins coûteuse
    - Traductions de référence
    - Similarité avec les traductions de référence:
    - Différentes mesures: BLEU, NIST, Précision/Rappel, METEOR

### BLEU

- Nombre de n-grammes de différentes tailles en commun avec la traduction de référence. 
    - Mesure de précision modifiée: Moyenne des précisions pour tous les N-grammes jusqu'à N.
- Clip count: nombre maximum d'apparition d'un mot dans les références (peut être 1). Mesure de référence.

$$
p_n = \frac{\sum_{C \in corpus}\sum_{ngram \in C} count_{clip}(ngram)}{\sum_{C \in corpus}\sum_{ngram \in C} count(ngram)}
$$

$$
p = \sqrt[N]{\prod_{n=1}^N p_n}
$$

- Pénalité pour les phrases courtes:
    - c = longueur de la traduction
    - r = longueur de la phrase de référence ayant le plus grand nombre de n-grammes en commun avec la traduction

$$
BP = 1_{c>r} + e^{1-r/c}*1_{c \leq r}
$$

# Introduction aux réseaux de neurones

## Trois visions du réseau de neurones:

- Métaphore de la structure du cerveau. Un neurone est composé de:
    - Dendrites
    - Noyau
    - Axone
    - Synapses
- Une cascade de régressions logistiques
    - Somme pondérée des valeurs en entrée (signal z)
    - Fonction d'activation sigmoide
    - Signal en sortie

    $$
    y = \sigma(w \cdot x + b) = (1+\exp(-(w \cdot x + b)))^{-1}
    $$
- Un large réseau d'opérations simples
    - Différentes fonctions d'activation
        - sigmoide: $\frac{1}{1-e^{-x}}$
        - tanh: $\frac{e^x-e^{-x}}{e^x+e^{-x}}$
        - RELU: $max(x,0)$

## Réseau feedforward

- Unités connectées sans cycle
    - Plusieurs neurones = plus discriminant
    - Plusieurs séparateurs
- Pas de retour en arrière
- Couches:
    - Entrée
    - Cachées (représentation intermédiaire)
    - Sortie
- Notation matricielle:

$$
\begin{aligned}
\mathbf{W} &= \begin{pmatrix}{w_{1,1}} & {w_{1,2}} & {w_{1,3}} & \cdots & {w_{1,n_0}}\\
{w_{2,1}} & {w_{2,2}} & {w_{2,3}} & \cdots & {w_{2,n_0}}\end{pmatrix}\\
\mathbf{b} &= \begin{pmatrix}{b_{h,1}} & {b_{h,2}} & {b_{h,3}} & \cdots & {b_{h,n,1}}\end{pmatrix}\\
\mathbf{h} &= \begin{pmatrix}{a_{h,1}} & {a_{h,2}} & {a_{h,3}} & \cdots & {a_{h,n,1}}\end{pmatrix}
\mathbf{h} &= \sigma(\mathbf{W}x+\mathbf{b})\\
\mathbf{U} &= \begin{pmatrix}{u_{1,1}} & {u_{1,2}} & {u_{1,3}} & \cdots & {u_{1,n_0}}\\
{u_{2,1}} & {u_{2,2}} & {u_{2,3}} & \cdots & {u_{2,n_0}}\end{pmatrix}\\
\mathbf{z} &= \mathbf{U}h \\
\mathbf{y} &= softmax(\mathbf{z}) \\
&=  \begin{pmatrix}{\frac{e^{z_1}}{\sum_{j=1}^d e^{z_j}}} & \cdots & {\frac{e^{z_d}}{\sum_{j=1}^d e^{z_j}}}\end{pmatrix}
\end{aligned}
$$

On veut minimiser une fonction de perte $L$ à l'aide des poids $w_i$ et des biais $b$.

- Rétropropagation des erreurs
- Descente de gradient

## Utilisation en NLP

Classificateurs supervisés:

- Segmentation de phrases
- Identification de la langue
- Analyse de sentiments (classificateur binaire)
- Extraction de relations

Approches:

- Sélection automatique des attributs
- Prétraitement des textes pour construire des attributs
    - Sac de mots
    - Mots négatifs
    - Entités nommées
- Classificateur supervisé sur ces attributs

**Apprentissage de la représentation** (*representation learning*): 

- Le classificateur utilisé précédemment est remplacé par un réseau de neurones.
- On fournit un texte brut au réseau et on lui laisse découvrir les bons attributs.
- Une bonne représentation des bots: plongements de mots (*word embeddings*) 

# Plongements de mots



# Modèles et applications






